<?php
	/**************************************************************************\
 	* phpGroupWare API - Accounts manager for LDAP                             *
	* Written by Joseph Engo <jengo@phpgroupware.org>                          *
	*        and Lars Kneschke <lkneschke@phpgw.de>                            *
	*        and Bettina Gille <ceb@phpgroupware.org>                          *
	*        and Philipp Kamps <pkamps@probusiness.de>                         *
	* View and manipulate account records using LDAP                           *
	* Copyright (C) 2000 - 2002 Joseph Engo, Lars Kneschke                     *
	* Copyright (C) 2003 Lars Kneschke, Bettina Gille                          *
	* ------------------------------------------------------------------------ *
	* This library is part of the phpGroupWare API                             *
	* http://www.phpgroupware.org                                              * 
	* ------------------------------------------------------------------------ *
	* This library is free software; you can redistribute it and/or modify it  *
	* under the terms of the GNU Lesser General Public License as published by *
	* the Free Software Foundation; either version 2.1 of the License,         *
	* or any later version.                                                    *
	* This library is distributed in the hope that it will be useful, but      *
	* WITHOUT ANY WARRANTY; without even the implied warranty of               *
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
	* See the GNU Lesser General Public License for more details.              *
	* You should have received a copy of the GNU Lesser General Public License *
	* along with this library; if not, write to the Free Software Foundation,  *
	* Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA            *
	\**************************************************************************/
	/* $Id: class.accounts_ldap.inc.php,v 1.101.2.13.2.16 2004/01/09 12:43:17 skwashd Exp $ */

	class accounts extends accounts_
	{
		var $ds;
		var $user_context  = '';
		var $group_context = '';

		//Be Carefull to correctly set the following ! 
		//Need to be set in the setup system ! (Caeies)
		var $rdn_account = 'uid';
		var $rdn_group   = 'cn';
		var $fallback_homedirectory = '/dev/null'; //Be secure by default ;) (Caeies)
		var $fallback_loginshell    = '/bin/false'; //Nothing allowed ;) (Caeies)
		var $fallback_apps          = array('addressbook',
		                                    'calendar',
		                                    'email',
					                        'notes',
					                        'todo',
					                        'phpwebhosting', //isn't filemanager ?? (Caeies)
					                        'manual'
					                              );

		function accounts($account_id = '', $account_type = '')
		{
			if($GLOBALS['phpgw_info']['server']['account_repository'] == 'ldap')
			{
				$this->ds = $GLOBALS['phpgw']->common->ldapConnect();
				$this->user_context  = $GLOBALS['phpgw_info']['server']['ldap_context'];
				$this->group_context = $GLOBALS['phpgw_info']['server']['ldap_group_context'];
			}
			$this->accounts_($account_id, $account_type);
		}

		function read_repository()
		{

			$acct_type = $this->get_type($this->account_id);

			/* search the dn for the given uid */
			if ( ($acct_type == 'g') && $this->group_context )
			{
				$sri = ldap_search($this->ds, $this->group_context, 'gidnumber='.$this->account_id);
			}
			else
			{
				$sri = ldap_search($this->ds, $this->user_context, 'uidnumber='.$this->account_id);
			}
			$allValues = ldap_get_entries($this->ds, $sri);

			/* Now dump it into the array; take first entry found */
			if($acct_type == 'g')
			{
				$this->data['account_id']           = $allValues[0]['gidnumber'][0];
				$this->data['account_lid']          = $allValues[0]['cn'][0];
				$this->data['account_firstname']    = utf8_decode($allValues[0]['cn'][0]);
				$this->data['account_lastname']     = 'Group';
				$this->data['type']                 = 'g';
			}
			else
			{
				$this->data['account_id']           = $allValues[0]['uidnumber'][0];
				$this->data['account_lid']          = $allValues[0]['uid'][0];
				$this->data['account_firstname']    = utf8_decode($allValues[0]['givenname'][0]);
				$this->data['account_lastname']     = utf8_decode($allValues[0]['sn'][0]);
				$this->data['type']                 = 'u';
			}
			$this->data['account_dn']             = $allValues[0]['dn'];
			$this->data['fullname']               = $allValues[0]['cn'][0];

			$this->data['homedirectory']          = $allValues[0]['homedirectory'][0];
			$this->data['loginshell']             = $allValues[0]['loginshell'][0];

			$this->data['expires']                = $allValues[0]['accountexpires'][0];
			if ($allValues[0]['accountstatus'][0] == 'A')
			{
				$this->data['status'] = 'A';
			}
			return $this->data;
		}

		/* ohne worte */
		function save_repository()
		{
			$acct_type = $this->get_type($this->account_id);

			if ($acct_type == 'g')
			{
				return $this->create_group($this->data, '');
			}
			else
			{
				return $this->create_account($this->data, '');			
			}
		}
		
		/**
		 * delete an account or group of the phpgw
		 *
		 * @param integer $id id of group/account to delete
		 * @return boolean success of delete 
		 */
		function delete($id = '')
		{
			$id = get_account_id($id);
			$type = $this->get_type($id);

			if ($type == 'g')
			{
				$sri = ldap_search($this->ds, $this->group_context, '(&(objectclass=phpgwgroup)(gidnumber='.$id.'))');
				$allValues = ldap_get_entries($this->ds, $sri);
			}
			else
			{
				$sri = ldap_search($this->ds, $this->user_context, '(&(objectclass=phpgwaccount)(uidnumber='.$id.'))');
				$allValues = ldap_get_entries($this->ds, $sri);
			}
				
			if ($allValues[0]['dn'])
			{
				$del = ldap_delete($this->ds, $allValues[0]['dn']);
				return $del; 
			}
			else
			{
				return false;
			}
		}

		/**
		 * reads all accounts or groups for the phpgw
		 *
		 * @param string 
		 * @param string
		 * @param string
		 * @param string
		 * @param string
		 * @param string
		 * @return mixed 
		 */
		function get_list($_type='both', $start = '',$sort = '', $order = '', $query = '', $offset = '')
		{
			$query = strtolower($query);
			if ($offset)
			{
				$limitclause = '';//$phpgw->db->limit($start,$offset);
			}
			elseif ($start && !$offset)
			{
				$limitclause = '';//$phpgw->db->limit($start);
			}

			if (! $sort)
			{
				$sort = '';//"desc";
			}

			if ($_type == 'accounts')
			{
				$listentries = $this->get_accountList($query);
			}
			elseif ($_type == 'groups')
			{
				$listentries = $this->get_groupList($query);
			}
			else
			{
				$listentries = array_merge($this->get_accountList($query), $this->get_groupList($query));
			}

			// sort the array
			$arrayFunctions = CreateObject('phpgwapi.arrayfunctions');
			if(empty($order))
			{
				$order = 'account_lid';
			}
			$sortedlist = $arrayFunctions->arfsort($listentries,array($order),$sort);
			$this->total = count($listentries); // this shouldn't be an obejct var for one account/group whatever

			if (is_array($sortedlist))
			{
				reset($sortedlist);
				if(is_int($start) && is_int($offset))
				{
					return array_slice($sortedlist, $start, $offset);
				}
				elseif(is_int($start))
				{
					return array_slice($sortedlist, $start, $GLOBALS['phpgw_info']['user']['preferences']['common']['maxmatchs']);
				}
				else
				{
					return $sortedlist;
				}
			}
			return false;
		}
		
		/**
		 * reads all accounts for the phpgw
		 *
		 * @param string $query
		 * @return array Array of accounts
		 */
		function get_accountList($query)
		{
			if(empty($query) || $query == "*")
			{
				$filter = '(&(uidnumber=*)(objectclass=posixaccount))';
			}
			else
			{
				$filter = "(&(uidnumber=*)(objectclass=posixaccount)(|(uid=*$query*)(sn=*$query*)(cn=*$query*)(givenname=*$query*)))";
			}
			$sri = ldap_search($this->ds, $this->user_context, $filter);
			$allValues = ldap_get_entries($this->ds, $sri);
			while (list($null,$allVals) = @each($allValues))
			{
				settype($allVals,'array');
				$test = @$allVals['uid'][0];
				if (!$GLOBALS['phpgw_info']['server']['global_denied_users'][$test] && $allVals['uid'][0])
				{
					$accounts[] = Array(
						'account_id'        => $allVals['uidnumber'][0],
						'account_lid'       => $allVals['uid'][0],
						'account_type'      => 'u',
						'account_firstname' => utf8_decode($allVals['givenname'][0]),
						'account_lastname'  => utf8_decode($allVals['sn'][0]),
						'account_status'    => $allVals['accountstatus'][0]
					);
				}
			}
			return $accounts;
		}

		/**
		 * reads all groups for the phpgw
		 *
		 * @param string $query
		 * @return array Array of groups
		 */
		function get_groupList($query)
		{
			if(empty($query) || $query == "*")
			{
				$filter = '(&(gidnumber=*)(objectclass=posixgroup))';
			}
			else
			{
				$filter = "(&(gidnumber=*)(objectclass=posixgroup)(|(uid=*$query*)(sn=*$query*)(cn=*$query*)(givenname=*$query*)))";
			}
			$sri = ldap_search($this->ds, $this->group_context, $filter);
			$allValues = ldap_get_entries($this->ds, $sri);
			while (list($null,$allVals) = @each($allValues))
			{
				settype($allVals,'array');
				$test = $allVals['cn'][0];
				if (!$GLOBALS['phpgw_info']['server']['global_denied_groups'][$test] && $allVals['cn'][0])
				{
					$groups[] = Array(
						'account_id'        => $allVals['gidnumber'][0],
						'account_lid'       => $allVals['cn'][0],
						'account_type'      => 'g',
					);
				}
			}
			return $groups;
		}

		/**
		 * gets the id of a group or an account
		 *
		 * @param array $name group name or login name the method search the id for
		 * @return mixed id of the group/account when found
		 *               false when group/account name do not exist
		 */
		function name2id($account_lid)
		{
			static $name_list;

			if(@isset($name_list[$account_lid]) && $name_list[$account_lid])
			{
				return $name_list[$account_lid];
			}

			$id  = $this->groupName2id($account_lid);
			$uid = $this->accountName2id($account_lid);
			
			if ($uid)
			{
				$id = $uid;
			}
			return $id;
		}

		/**
		 * gets the id to a group name
		 *
		 * @param array $name name of the group the method search the id
		 * @return mixed id of the group when found
		 *               false when group name do not exist
		 */
		function groupName2id ($name)
		{
			$sri = ldap_search($this->ds, $this->group_context, '(&(cn='. $name .')(objectclass=phpgwgroup))');
			$allValues = ldap_get_entries($this->ds, $sri);
			
			if (@$allValues[0]['gidnumber'][0])
			{
				return intval($allValues[0]['gidnumber'][0]);
			}
			else
			{
				return false;
			}
		}
		
		/**
		 * gets the id to a account name
		 *
		 * @param array $name login name of the account the method search the id
		 * @return mixed id of the account when found
		 *               false when account name does not exist
		 */
		function accountName2id($name)
		{
			$sri = ldap_search($this->ds, $this->user_context, '(&(uid='. $name .')(objectclass=phpgwaccount))');
			$allValues = ldap_get_entries($this->ds, $sri);

			if (@$allValues[0]['uidnumber'][0])
			{
				return intval($allValues[0]['uidnumber'][0]);
			}
			else
			{
				return false;
			}
		}

		/**
		 * gets the name to an account or group id
		 *
		 * @param integer $id id of the account or the group the method search the name
		 * @return mixed name of the account or the group when found
		 *               false when account or group name do not exist
		 */
		function id2name($id)
		{
			static $id_list;
			if(isset($id_list[$id]))
			{
				return $id_list[$id];
			}

			$type = $this->get_type($id);
			
			if ($type == 'g')
			{
				$group = $this->group_exists($id);
				$name = $group['cn'][0];
			}
			elseif ($type == 'u')
			{
				$account = $this->account_exists($id);
				$name = $account['uid'][0];
			}
			else
			{
				return false;
			}
			return $name;
		}

 		function search_person($person_id)
		{
			static $person_list;
			if(isset($person_list[$person_id]))
			{
				return intval($person_list[$person_id]);
			}

			$allValues = array();
			// Groups are person? are you sure?
			$sri = ldap_search($this->ds, $this->group_context, "(&(person_id=$person_id)(phpgwaccounttype=g))");
			$allValues = ldap_get_entries($this->ds, $sri);

			if (@$allValues[0]['gidnumber'][0])
			{
				$person_list[$person_id] = intval($allValues[0]['gidnumber'][0]);
				return $person_list[$person_id];
			}

			$allValues = array();
			$sri = ldap_search($this->ds, $this->user_context, "(&(person_id=$person_id)(phpgwaccounttype=u))");
			$allValues = ldap_get_entries($this->ds, $sri);

			if (@$allValues[0]['uidnumber'][0])
			{
				$id_list[$person_id] = intval($allValues[0]['uidnumber'][0]);
				return $person_list[$person_id];
			}

			return $person_list[$person_id];
		}

		/**
		 * tries to guess the if the given id is a group or an account
		 *
		 * @param integer $id
		 * @return mixed 'u' for account (user)
		 *               'g' for group
		 *               'false' for no existing id
		 */
		function get_type($id = '') // get_type() without an id - what do you expect me to return!?
		{
			if ($this->account_exists($id))
			{
				$type = 'u';
			}
			if ($this->group_exists($id))
			{
				if ($type == 'u')
				{
					die('account/group id conflict - bad luck');
				}
				else
				{
					$type = 'g';
				}
			}
			if ($type)
			{
				return $type;
			}
			else
			{
				return false;
			}
		}

		

		/**
		 * gets next free id for an account or for a group
		 *
		 * @param byte $type 'u' for account (user), 'g' for group
		 * @return mixed free id for an account or a group
		 *               false when no free id is available
		 */
		function get_nextid($type = 'u')
		{
			if ($type == 'u')
			{
				return $this->get_nextAccountID();
			}
			elseif ($type == 'g')
			{
				return $this->get_nextGroupID();
			}
			else
			{
				return false;
			}
		}
		
		/**
		 * gets next free id for an account
		 *
		 * @return mixed free id for an account
		 *               false when no free id is available
		 */
		function get_nextAccountID()
		{
			$filter = '(|(objectclass=posixaccount)(objectclass=phpgwaccount))';
			$result = ldap_search($this->ds, $this->user_context, $filter, array('uidnumber'));
			$allValues = ldap_get_entries($this->ds, $result);

			// parse all LDAP uidnumbers in a single array '$IDs'
			while (list($key,$val) = each($allValues))
			{
				if (is_int($key))
				{
					$IDs[] = $allValues[$key]['uidnumber'][0];
				}
			}
			return $this->idTester($IDs, $GLOBALS['phpgw_info']['server']['account_min_id'], $GLOBALS['phpgw_info']['server']['account_max_id'] );
		}
		
		/**
		 * gets next free id for a group
		 *
		 * @return mixed free id for a group
		 *               false when no free id is available
		 */
		function get_nextGroupID()
		{
			$filter = '(|(objectclass=posixgroup)(objectclass=phpgwgroup))';
			$result = ldap_search($this->ds, $this->group_context, $filter, array('gidnumber'));
			$allValues = ldap_get_entries($this->ds, $result);

			while (list($key,$val) = each($allValues))
			{
				if (is_int($key))
				{
					$IDs[] = $allValues[$key]['gidnumber'][0];
				}
			}
			return $this->idTester($IDs, $GLOBALS['phpgw_info']['server']['group_min_id'], $GLOBALS['phpgw_info']['server']['group_max_id'] );
		}

		/**
		 * checks if a group exists
		 *
		 * @param integer $id
		 * @param string $dn
		 * @return mixed 
		 */
		function group_exists($id, $dn = '')
		{
			if ($id)
			{
				$result = ldap_search($this->ds, $this->group_context, 'gidnumber=' . $id);
				$allValues = ldap_get_entries($this->ds, $result);
				if ($allValues[0]['dn'])
				{
					return $allValues[0];
				}
				else
				{
					return $this->dn_exists($dn);
				}
			}
			return false;
		}
		
		/**
		 * checks if an account exists
		 *
		 * @param integer $id
		 * @param string $dn
		 * @return mixed 
		 */
		function account_exists($id, $dn = '')
		{
			if ($id)
			{
				$result = ldap_search($this->ds, $this->user_context, 'uidnumber=' . $id);
				$allValues = ldap_get_entries($this->ds, $result);
				if ($allValues[0]['dn'])
				{
					return $allValues[0];
				}
				else
				{
					return $this->dn_exists($dn);
				}
			}
			return false;
		}

		/**
		 * checks if an phpgwcontact exists in LDAP
		 *
		 * @param integer $id
		 * @param string $dn
		 * @return mixed 
		 */
		function person_exists($id, $dn = '')
		{
			if ($id)
			{
				$result = ldap_search($this->ds, $this->user_context, 'phpgwcontact=' . $id);
				$allValues = ldap_get_entries($this->ds, $result);
				if ($allValues[0]['dn'])
				{
					return $allValues[0];
				}
				else
				{
					return $this->dn_exists($dn);
				}
			}
			return false;
		}

		
		/**
		 * checks if the given dn exists
		 *
		 * @param string $dn
		 * @return mixed 
		 */
		function dn_exists($dn)
		{
			if ($dn != '')
			{
				$result = @ldap_search($this->ds, $dn, 'objectclass=*');
				if ($result)
				{
					$allValues = ldap_get_entries($this->ds, $result);
					if ($allValues[0]['dn'])
					{
						return $allValues[0];
					}
				}
			}
			return false;
		}

		/**
		 * checks if a group or an account exists
		 *
		 * @param integer $id
		 */
		function exists($id)
		{
			if (!is_int($id) && $id != '')
			{
				$id = $this->name2id($id);
			}

			if ($id)
			{
				$return = 0;
				if ($this->account_exists($id))
				{
					$return++;
				}
				if ($this->group_exists($id));
				{
					$return++;
				}
				return $return;
			}
			else
			{
				return false;
			}
		}

		/**
		 * creates an account or group
		 *
		 * @param array $account_info
		 * @param string $default_prefs
		 * @return mixed 
		 */
		function create($account_info,$default_prefs = true)
		{
			if (empty($account_info['account_id']) || !$account_info['account_id'])
			{
				$account_info['account_id'] = $this->get_nextid($account_info['account_type']);
			}
			if ($account_info['account_type'] == 'u')
			{
				$this->create_account($account_info, $default_prefs);
			}
			elseif($account_info['account_type'] == 'g')
			{
				$this->create_group($account_info, $default_prefs);
			}
			else
			{
				return false;
			}
			
			if($account_info['account_id'] && is_object($GLOBALS['phpgw']->preferences) && $default_prefs)
			{
				$GLOBALS['phpgw']->preferences->create_defaults($account_info['account_id']);
			}
			return $account_info['account_id'];
		}

		/**
		 * creates an account
		 *
		 * @param array $account_info
		 * @param string $default_prefs
		 * @return mixed 
		 */
		function create_account($account_info, $default_prefs)
		{
			$dn = $this->rdn_account .
			      '=' .
			      $this->get_leafName($account_info['account_firstname'], $account_info['account_lastname'], $account_info['account_lid']) .
			      ',' .
			      $this->user_context;
				
			// phpgw attributes
			$entry['objectclass'][]       = 'phpgwAccount';
			$entry['phpgwAccountID']      = $account_info['account_id'];
			$entry['accountexpires']      = $account_info['account_expires'];
			if ($account_info['account_status'])
			{
				$entry['accountstatus'] = $account_info['account_status'];
			}
			else
			{
				$entry['accountstatus'] = 'I'; // 'I' for inactiv
			}
			if ($account_info['lastlogin'])
			{
				$entry['lastlogin'] = $account_info['lastlogin'];
			}
			if ($account_info['lastloginfrom'])
			{
				$entry['lastloginfrom'] = $account_info['lastloginfrom'];
			}
			if ($account_info['lastpasswd_change'])
			{
				$entry['lastpasswdchange'] = $account_info['lastpasswd_change'];
			}
				
			// additional attributes from the phpgw for groups
			$entry['objectclass'][]       = 'posixAccount';
			$entry['cn']                  = utf8_encode($this->get_fullname($account_info['account_firstname'], $account_info['account_lastname']));
			$entry['uidnumber']           = $account_info['account_id'];
			$entry['uid']                 = $account_info['account_lid'];
			// There is a problem here if an * is present :) (Caeies)
			$entry['description']         = str_replace('*', '', lang('phpgw-created account'));
			if ( $account_info['account_firstname'] )
			{
				$entry['givenname'] = utf8_encode($account_info['account_firstname']);
			}
			if ( $account_info['account_lastname'] )
			{
				$entry['sn'] = utf8_encode($account_info['account_lastname']);
			}
			if ($account_info['account_passwd'])
			{
				$entry['userpassword'] = $GLOBALS['phpgw']->common->encrypt_password($account_info['account_passwd']);
			}

			//Object that are need by the constraint of facist LDAP
			//As fips say, it's not the right place here, but anyway, without it I'm not able to build an account (Caeies)
		//Be carefull to add the good things in the schema file (ie a SUP top AUXILIARY to phpgwGroup Objectclass
			$entry['objectclass'][]       = 'top';
			$entry['objectclass'][]       = 'person';
			$entry['objectclass'][]       = 'organizationalPerson';
			$entry['objectclass'][]       = 'inetOrgPerson';
			// Fields are must for LDAP - so we write them in any case
			$entry['homedirectory']       = $this->get_homedirectory($account_info['homedirectory'], $account_info['account_lid']);
			// In fact the loginshell is not Required (Must) it's only optional (May) in posixAccount (Caeies), so don't need it now
			//$entry['loginshell']          = $this->get_loginshell($account_info['loginshell']);

				
			// special gidnumber handling
			if ($GLOBALS['phpgw_info']['server']['ldap_group_id'])
			{
				$enty['gidnumber'] = $GLOBALS['phpgw_info']['server']['ldap_group_id'];
			}
			else
			{
				$entry['gidnumber']           = $account_info['account_id'];
			}

			$oldEntry = $this->account_exists($account_info['account_id'], $dn);
			
			if ($oldEntry) // found an existing entry in LDAP
			{
				if ($this->createMode == 'replace')
				{
					ldap_delete($this->ds, $oldEntry['dn']);
					$this->add_LDAP_Entry($dn, $entry);
				}
				elseif ($this->createMode == 'extend')
				{
					/* not yet implemented */
				}
				else  // createMode == 'modify'
				{
					while (list($key,$val) = each($oldEntry))
					{
						if (!is_int($key))
						{
							unset($oldEntry[$key]['count']);
							switch ($key)
							{
								case 'dn':
									if ($oldEntry['dn'] != $dn)  // new group name DN should renamed as well
									{
										$oldEntry['dn'] = $this->rename_LDAP_entry($oldEntry['dn'], $dn, $this->user_context);
										if (!$oldEntry)
										{
											print ldap_error($this->ds);
											die('<br><br>LDAP rename error');
										}											
									}
									break;
								
								case 'count':
								case 'cn':
								case 'description':
								case 'phpgwaccountid':
								case 'gidnumber':
								case 'accountstatus':
								case 'accountexpires':
								case 'uidnumber':
								case 'uid':
								case 'userpassword':
								case 'homedirectory':
								case 'loginshell':
								case 'givenname':
								case 'sn':
								case 'lastlogin':
								case 'lastloginfrom':
								case 'lastpasswordchange':
									break;
									
								case 'objectclass':
									if( !in_array('phpgwAccount', $oldEntry[$key]) && !in_array('phpgwAccount', $oldEntry[$key]) )
									{
										$entry[$key] = $oldEntry[$key];
										array_push($entry[$key], 'phpgwAccount');
									}
									else
									{
										$entry[$key] = $oldEntry[$key];
									}
									break;
										
								default:
									$entry[$key] = $oldEntry[$key];
							}
						} 
					}
					$this->modify_LDAP_Entry($oldEntry['dn'], $entry);
				}
			}
			else // entry not yet in LDAP
			{
				$this->add_LDAP_Entry($dn, $entry);
			}				
		}
		
		/**
		 * creates a group
		 *
		 * @param array $account_info
		 * @param string $default_prefs
		 * @return mixed 
		 */
		function create_group($account_info, $default_prefs)
		{
			$dn = $this->rdn_group . '=' . $account_info['account_lid'] . ',' . $this->group_context;

			// phpgw needed attributes
			
			$entry['objectclass'][]  = 'phpgwGroup';
			$entry['phpgwgroupID']   = $account_info['account_id'];
			$entry['gidnumber']      = $account_info['account_id'];				

			// additional attributes from the phpgw for groups
			$entry['objectclass'][]  = 'posixGroup';
			$entry['cn']             = $account_info['account_lid'];
			$entry['description']    = str_replace('*', '', lang('phpgw-created group'));
			$entry['memberuid']      = $this->get_memberUIDs($account_info['account_id']);
			if (!$entry['memberuid'])
			{
				unset ($entry['memberuid']);
			}

			$oldEntry = $this->group_exists($account_info['account_id'], $dn);

			if ($oldEntry) // found an existing entry in LDAP
			{
				if ($this->createMode == 'replace')
				{
					ldap_delete($this->ds, $oldEntry['dn']);
					$this->add_LDAP_Entry($dn, $entry);
				}
				elseif ($this->createMode == 'extend')
				{
					/* not yet implemented */
				}
				else  // createMode == 'modify'
				{
					while (list($key,$val) = each($oldEntry))
					{
						if (!is_int($key))
						{
							unset($oldEntry[$key]['count']);
							switch ($key)
							{
								case 'dn':
									if ($oldEntry['dn'] != $dn)  // new group name DN should renamed as well
									{
										$oldEntry['dn'] = $this->rename_LDAP_entry($oldEntry['dn'], $dn, $this->group_context);
										if (!$oldEntry)
										{
											print ldap_error($this->ds);
											die('<br><br>LDAP rename error');
										}											
									}
									break;
								case 'count':
								case 'cn':
								case 'description':
								case 'phpgwgroupid':
								case 'gidnumber':
								case 'memberuid':
									break;
									
								case 'objectclass':
									if( !in_array('phpgwGroup', $oldEntry[$key]) && !in_array('phpgwgroup', $oldEntry[$key]) )
									{
										$entry[$key] = $oldEntry[$key];
										array_push($entry[$key], 'phpgwGroup');
									}
									else
									{
											$entry[$key] = $oldEntry[$key];
									}
									break;
										
								default:
									$entry[$key] = $oldEntry[$key];
							}
						} 
					}
					$this->modify_LDAP_Entry($oldEntry['dn'], $entry);
				}
			}
			else // entry not yet in LDAP
			{
				$this->add_LDAP_Entry($dn, $entry);
			}				
		}
		
		/**
		 * add an entry to LDAP
		 *
		 * @param string $dn
		 * @param array $entry
		 * @return boolean 
		 */
		function add_LDAP_Entry($dn, $entry)
		{
			/*
			echo "<br><b>Adds: ".$dn."</b><br>";
			echo "<pre>";
			print_r($entry);
			echo "</pre>";
			echo "<br>";
			*/
			$success = ldap_add($this->ds, $dn, $entry);
			if (!$success)
			{
				print ldap_error($this->ds);
				die('die cause of LDAP error while adding');
			}
			else
			{
				return true;
			}
		}
		
		/**
		 * modify an entry in LDAP
		 *
		 * @param string $dn
		 * @param array $entry
		 * @return boolean 
		 */
		function modify_LDAP_Entry($dn, $entry)
		{
			/*
			echo "<br><b>Modifies: ".$dn."</b><br>";
			echo "<pre>";
			print_r($entry);
			echo "</pre>";
			echo "<br>";
			*/
			
			$success = ldap_modify($this->ds, $dn, $entry);
			if (!$success)
			{
				print ldap_error($this->ds);
				die('die cause of LDAP error while modifing');
			}
			else
			{
				return true;
			}
		}

		/**
		 * renames a LDAP entry
		 *
		 * @param string $oldDN 
		 * @param string $newDN 
		 * @param string $baseDN 
		 * @return boolean 
		 */
		function rename_LDAP_Entry($oldDN, $newDN, $baseDN)
		{
			$newDN_array = (ldap_explode_dn($newDN, 0));
			$oldDN_array = (ldap_explode_dn($oldDN, 0));

			unset($newDN_array['count']);
			unset($oldDN_array['count']);
										
			$newDN_RDN = $newDN_array[0];
			$oldDN_RDN = array_shift($oldDN_array);
			$oldDN_base  = implode(',', $oldDN_array);
			if ($newDN_RDN != $oldDN_RDN && $oldDN_base == $baseDN)
			{
				$success = ldap_rename ( $this->ds, $oldDN, $newDN_RDN, $baseDN, false);
				if ($success)
				{
					return $newDN;
				}
				else
				{
					return false;
				}
			}
		}
			

		/**
		 * this function creates a non existing but authorized user 
		 *
		 * @param string $accountname user name
		 * @param string $passwd user password
		 * @param boolean $default_prefs given default preferences for this new user
		 * @param boolean $default_acls given acls (modules) for this new user
		 * @param integer $expiredate expire date of this account. '-1' for never.
		 * @param char $account_status status for new user. 'A' for active user.
		 * @return integer accountID 
		 */
		function auto_add($accountname, $passwd, $default_prefs = false, $default_acls = false, $expiredate = 0, $account_status = 'A')
		{
			if ($expiredate)
			{
				$expires = mktime(2,0,0,date('n',$expiredate), intval(date('d',$expiredate)), date('Y',$expiredate));
			}
			else
			{
				if($GLOBALS['phpgw_info']['server']['auto_create_expire'])
				{
					if($GLOBALS['phpgw_info']['server']['auto_create_expire'] == 'never')
					{
						$expires = -1;
					}
					else
					{
						$expiredate = time() + $GLOBALS['phpgw_info']['server']['auto_create_expire'];
						$expires   = mktime(2,0,0,date('n',$expiredate), intval(date('d',$expiredate)), date('Y',$expiredate));
					}
				}
				else
				{
					/* expire in 30 days by default */
					$expiredate = time() + ( ( 60 * 60 ) * (30 * 24) );
					$expires   = mktime(2,0,0,date('n',$expiredate), intval(date('d',$expiredate)), date('Y',$expiredate));
				}
			}

			$acct_info = array(
				'account_id'        => $this->get_nextid('u'), //'u' for account
				'account_lid'       => $accountname,
				'account_passwd'    => $passwd,
				'account_firstname' => 'New',
				'account_lastname'  => 'User',
				'account_status'    => $account_status,
				'account_expires'   => $expires
			);
			$this->create_account($acct_info, $default_prefs);
			$accountid = $this->accountName2id($accountname);

			$this->db->transaction_begin();
			if ($default_acls == false)
			{
				$apps = $this->fallback_apps;
				
				$default_group_lid = $GLOBALS['phpgw_info']['server']['default_group_lid'];
				$default_group_id  = $this->groupName2id($default_group_lid);
				$defaultgroupid    = $default_group_id ? $default_group_id : $this->groupName2id('Default');
				if($defaultgroupid)
				{
					$this->db->query("INSERT INTO phpgw_acl (acl_appname, acl_location, acl_account, acl_rights) values('phpgw_group', "
						. $defaultgroupid . ', ' . $accountid . ', 1)',__LINE__,__FILE__);
				}
				$this->db->query("INSERT INTO phpgw_acl (acl_appname, acl_location, acl_account, acl_rights)values('preferences', 'changepassword', ".$accountid.", 1)",__LINE__,__FILE__);
				@reset($apps);
				while(list($key,$app) = each($apps))
				{
					$this->db->query("INSERT INTO phpgw_acl (acl_appname, acl_location, acl_account, acl_rights) VALUES ('".$app."', 'run', ".$accountid.", 1)",__LINE__,__FILE__);
				}
			}
			$this->db->transaction_commit();
			return $accountid;
		}

		function get_account_name($accountid,&$lid,&$fname,&$lname)
		{
			static $account_name;
			
			$account_id = get_account_id($accountid);
			if(isset($account_name[$account_id]))
			{
				$lid = $account_name[$account_id]['lid'];
				$fname = $account_name[$account_id]['fname'];
				$lname = $account_name[$account_id]['lname'];
				return;
			}
			$acct_type = $this->get_type($account_id);

			/* search the dn for the given uid */
			if ( ($acct_type == 'g') && $this->group_context )
			{
				$sri = ldap_search($this->ds, $this->group_context, 'gidnumber='.$account_id);
			}
			else
			{
				$sri = ldap_search($this->ds, $this->user_context, 'uidnumber='.$account_id);
			}
			$allValues = ldap_get_entries($this->ds, $sri);

			if($acct_type =='g')
			{
				$account_name[$account_id]['lid']   = $allValues[0]['cn'][0];
				$account_name[$account_id]['fname'] = $allValues[0]['cn'][0];
				$account_name[$account_id]['lname'] = 'Group';
			}
			else
			{
				$account_name[$account_id]['lid']   = $allValues[0]['uid'][0];
				$account_name[$account_id]['fname'] = $allValues[0]['givenname'][0];
				$account_name[$account_id]['lname'] = $allValues[0]['sn'][0];
			}
			$lid = $account_name[$account_id]['lid'];
			$fname = $account_name[$account_id]['fname'];
			$lname = $account_name[$account_id]['lname'];
			return;
		}

		function get_account_data($account_id)
		{
			$this->account_id = $account_id;
			$this->read_repository();

			$data[$this->data['account_id']]['lid']       = $this->data['account_lid'];
			$data[$this->data['account_id']]['firstname'] = $this->data['firstname'];
			$data[$this->data['account_id']]['lastname']  = $this->data['lastname'];
			$data[$this->data['account_id']]['fullname']  = $this->data['fullname'];
			$data[$this->data['account_id']]['type']      = $this->data['account_type'];
			$data[$this->data['account_id']]['person_id'] = $this->data['person_id'];
			return $data;
		}

		/**
		 * gets the DN for the given account id 
		 *
		 * @param interger $id
		 * @return mixed DN string when DN exists
		 *               false when DN doesn't exist
		 */
		function getDNforID($id = '') // ;-) I love to see that $id is optional
		{
			return $this->getDNforAccountID($_accountid); 
		}

		/**
		 * gets the DN for the given account id 
		 *
		 * @param interger $id
		 * @return mixed DN string when DN exists
		 *               false when DN doesn't exist
		 */
		function getDNforAccountID($_accountid = '')
		{
			$_account_id = get_account_id($id);

			$sri = ldap_search($this->ds, $this->user_context, 'uidnumber='.$id, array('dn'));
			$allValues = ldap_get_entries($this->ds, $sri);
			if ($allValues[0]['dn'])
			{
				return $allValues[0]['dn'];
			}
			else
			{
				return false;
			}
		}

		function get_account_with_contact()
		{
			$sri = ldap_search($this->ds, $this->user_context, "(&(phpgwaccounttype=u)(phpgwpersonid=*))", array('uidnumber', 'phpgwpersonid'));
			$allValues = ldap_get_entries($this->ds, $sri);
			if(is_array($allValues))
			{
				foreach($allValues as $value)
				{
					$accounts[$value['uidnumber'][0]] = $value['phpgwpersonid'][0];
				}
			}
			
			return $accounts;
		}

		function get_account_without_contact()
		{
			$sri = ldap_search($this->ds, $this->user_context, "(&(phpgwaccounttype=u)(!(phpgwpersonid=*)))", array('uidnumber'));
			$allValues = ldap_get_entries($this->ds, $sri);
			if(is_array($allValues))
			{
				foreach($allValues as $value)
				{
					$accounts[] = $value['uidnumber'][0];
				}
			}
			return $accounts;
		}
		
		/**
		 * dn leaf name generation
		 *
		 * @param string $first
		 * @param string $last
		 * @param string $login
		 * @return string generated name of leaf of the DN
		 */
		function get_leafName($first, $last, $login)
		{
			//if ($first != '' && $last == '')
			{
				$return = $login;
			}
			//else
			{
				//$return = $first.' '.$last;  
			}
			return $return;
		}
		
		/**
		 * full name generation
		 *
		 * @param string $first
		 * @param string $last
		 * @return string generated the fullname
		 */
		function get_fullname($first, $last)
		{
			return $first.' '.$last;
		}
		
		/**
		 * test an array with ids to look for a free id in respect to a min and max id
		 *
		 * @param array $IDs
		 * @param integer $min
		 ' @param max $max
		 * @return integer free id
		 */
		function idTester($IDs, $min = 1, $max = 0)
		{
		
			$interval = intval($max) - intval($min);
			$base = intval($min) ? intval($min) : 1;
			if ($interval > 0)
			{
				$pointer = $start = $IDs[count($IDs)-1]%$interval;
			}
			else
			{
				$pointer = $IDs[count($IDs)-1];
			}
			if (is_array($IDs))
			{
				do
				{
					if ($interval > 0)
					{
						$pointer = ($pointer+1)%$interval;
						if ($pointer == $start)
						{
							die('no free id found');
						}
					}
					else
					{
						$pointer++;
					}
				}
				while (in_array($base + $pointer, $IDs) && $pointer != $start);
			}
			else
			{
				$pointer = 0;
			}
			return $base + $pointer;
		}
		
		function get_homedirectory($newValue, $login)
		{
			if ($newValue != '' && $newValue != $GLOBALS['phpgw_info']['server']['ldap_account_home'])
			{
				$return = $newValue;
			}
			else
			{
				if ($GLOBALS['phpgw_info']['server']['ldap_account_home'] != '')
				{
					$return = $GLOBALS['phpgw_info']['server']['ldap_account_home'].SEP.$login;
				}
				else
				{
					$return = $this->fallback_homedirectory;
				}
			}
			return $return;
		}
		
		function get_loginshell($newValue)
		{
			if ($newValue != '')
			{
				$return = $newValue;
			}
			else
			{
				if ($GLOBALS['phpgw_info']['server']['ldap_account_shell'] != '')
				{
					$return = $GLOBALS['phpgw_info']['server']['ldap_account_shell'];
				}
				else
				{
					$return = $this->fallback_loginshell;
				}
			}
			return $return;
		}
		
		function get_memberUIDs($account_id = '')
		{
			if ( !empty($account_id) )
			{
   			$members = $this->member($account_id);
			}
			else
			{
   			$members = $this->member($this->data['account_id']);
			}
			$return = array();
			for ($i=0; $i<count($members); $i++)
			{
				$member = $this->id2name($members[$i]['account_id']);
				// function $this->member returns duplicated entries and empty entries :-(
				if (!in_array($member, $return) && $member != '') 
				{
					$return[] = $member;
				}
			}
			if (count($return))
			{
				return $return;
			}
			else
			{
				return false;
			}
		}

		/**
		 * adds an account to a group entry by adding the account name to the memberuid attribute
		 *
		 * @param integer $accountID
		 * @param integer $groupID
		 * @return boolean sucess
		 */
		function add_account2Group($accountID, $groupID)
		{
			if ($accountID && $groupID)
			{
				$groupEntry = $this->group_exists($groupID);
				$memberUID = $this->id2name($accountID);
				if ($groupEntry && $memberUID)
				{
					if (!is_array($groupEntry['memberuid']) || !in_array($memberUID, $groupEntry['memberuid']))
					{
						$entry['memberuid'][] = $memberUID;
						return ldap_mod_add($this->ds, $groupEntry['dn'], $entry); 
					}
				}
			}
			return false;
		}
		
		/**
		 * deletes an account for a group entry by removing the account name from the memberuid attribute
		 *
		 * @param integer $accountID
		 * @param integer $groupID
		 * @return boolean success
		 */
		function delete_account4Group($accountID, $groupID)
		{
			if ($accountID && $groupID)
			{
				$groupEntry = $this->group_exists($groupID);
				$memberUID = $this->id2name($accountID);
				if ($groupEntry && $memberUID)
				{
					if (is_array($groupEntry['memberuid']))
					{
						for ($i=0; $i < count($groupEntry['memberuid']); $i++)
						{
							if ($groupEntry['memberuid'][$i] == $memberUID)
							{
								$entry['memberuid'][] = $memberUID;
								return ldap_mod_del($this->ds, $groupEntry['dn'], $entry);
							}
						}
					}
				}
			}
			return false;
		}
	}
