<?php
  /**************************************************************************\
  * phpGroupWare API - VCard import/export class                             *
  * This file written by Miles Lott <milosch@phpgroupware.org>               *
  * Parse vcards->contacts class fields, and vice-versa                      *
  * Copyright (C) 2001 Miles Lott                                            *
  * -------------------------------------------------------------------------*
  * This library is part of the phpGroupWare API                             *
  * http://www.phpgroupware.org/api                                          * 
  * ------------------------------------------------------------------------ *
  * This library is free software; you can redistribute it and/or modify it  *
  * under the terms of the GNU Lesser General Public License as published by *
  * the Free Software Foundation; either version 2.1 of the License,         *
  * or any later version.                                                    *
  * This library is distributed in the hope that it will be useful, but      *
  * WITHOUT ANY WARRANTY; without even the implied warranty of               *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
  * See the GNU Lesser General Public License for more details.              *
  * You should have received a copy of the GNU Lesser General Public License *
  * along with this library; if not, write to the Free Software Foundation,  *
  * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA            *
  \**************************************************************************/

  /* $Id: class.vcard.inc.php,v 1.4.4.2 2003/11/03 20:25:02 alexbsa Exp $ */

	class vcard
	{
		var $import = array(
			'n'        => 'n',
			'sound'    => 'sound',
			'bday'     => 'bday',
			'note'     => 'note',
			'tz'       => 'tz',
			'geo'      => 'geo',
			'url'      => 'url',
			'pubkey'   => 'pubkey',
			'org'      => 'org',
			'title'    => 'title',
			'adr'      => 'adr',
			'label'    => 'label',
			'tel'      => 'tel',
			'email'    => 'email'
		);

		var $export = array(
			'full_name'                  => 'FN',
			'first_name'             => 'N;GIVEN',
			'last_name'            => 'N;FAMILY',
			'middle_name'            => 'N;MIDDLE',
			'prefix'            => 'N;PREFIX',
			'suffix'            => 'N;SUFFIX',
			'sound'               => 'SOUND',
			'birthday'                => 'BDAY',
			'note'                => 'NOTE',
			'tz'                  => 'TZ',
			'geo'                 => 'GEO',
			'url'                 => 'URL',
			'pubkey'              => 'PUBKEY',
			'org_name'            => 'ORG;NAME',
			'org_unit'            => 'ORG;UNIT',
			'title'               => 'TITLE',

			'adr_one_type'        => 'ADR;TYPE;WORK',
			'adr_two_type'        => 'ADR;TYPE;HOME',
			'tel_prefer'          => 'TEL;PREFER',
			'email_type'          => 'EMAIL;TYPE;WORK',
			'email_home_type'     => 'EMAIL;TYPE;HOME',

			'adr_one_street'      => 'ADR;WORK;STREET',
			'adr_one_locality'    => 'ADR;WORK;LOCALITY', 
			'adr_one_region'      => 'ADR;WORK;REGION', 
			'adr_one_postalcode'  => 'ADR;WORK;POSTALCODE',
			'adr_one_countryname' => 'ADR;WORK;COUNTRYNAME',
			'address2'            => 'EXT',
			'label'               => 'LABEL',

			'adr_two_street'      => 'ADR;HOME;STREET',
			'adr_two_locality'    => 'ADR;HOME;LOCALITY',
			'adr_two_region'      => 'ADR;HOME;REGION',
			'adr_two_postalcode'  => 'ADR;HOME;POSTALCODE',
			'adr_two_countryname' => 'ADR;HOME;COUNTRYNAME',

			'tel_work'            => 'TEL;WORK',
			'tel_home'            => 'TEL;HOME',
			'tel_voice'           => 'TEL;VOICE',
			'tel_fax'             => 'TEL;FAX',
			'tel_msg'             => 'TEL;MSG',
			'tel_cell'            => 'TEL;CELL',
			'tel_pager'           => 'TEL;PAGER',
			'tel_bbs'             => 'TEL;BBS',
			'tel_modem'           => 'TEL;MODEM',
			'tel_car'             => 'TEL;CAR',
			'tel_isdn'            => 'TEL;ISDN',
			'tel_video'           => 'TEL;VIDEO',
			'email'               => 'EMAIL;WORK',
			'email_home'          => 'EMAIL;HOME'
		);

		var $names = array(
			'first_name' => 'first_name',
			'last_name'  => 'last_name',
			'middle_name' => 'middle_name',
			'prefix' => 'prefix',
			'suffix' => 'suffix'
		);

		function vcard()
		{
			/* _debug_array($this); */
		}

		/*
			This now takes the upload filename
			and parses using the class string and array processors.
			The return is a contacts class entry, ready for add.
		*/
		function in_file($filename='')
		{
			if (!$filename)
			{
				return array();
			}

			$buffer = array();
			
			$fp = fopen($filename,'r');
			while ($data = fgets($fp,8000))
			{
				list($name,$value,$extra) = split(':', $data);
				if (substr($value,0,5) == 'http')
				{
					$value = $value . ':'.$extra;
				}
				if ($name && $value)
				{
					reset($this->import);
					while ( list($fname,$fvalue) = each($this->import) )
					{
						if ( strstr(strtolower($name), $this->import[$fname]) )
						{
							$value = trim($value);
							$value = ereg_replace("=0D=0A","\n",$value);
							$buffer += array($name => $value);
						}
					}
				}
			}
			fclose($fp);

			$entry = $this->in($buffer);
			//var_dump($entry);

			/* _debug_array($entry);exit; */

			return $entry;
		}

		/*
			This is here to match the old in() function, now called _parse_in().
			It is called now also by in_file() above.
			It takes a pre-parsed file using the methods in in_file(), returns a clean contacts class array.
		*/
		function in($buffer)
		{
			$buffer = $this->_parse_in($buffer);
			return $buffer;			
		}

		/*
			Pass this an associative array of fieldnames and values
			returns a clean array based on contacts class std fields
			This array can then be passed via $phpgw->contacts->add($ownerid,$buffer)
		*/
		function _parse_in($buffer)
		{
			/* Following is a lot of pain and little magic */
			while ( list($name,$value) = @each($buffer) )
			{
				$field  = split(';',$name);
				$field[0] = ereg_replace("A\.",'',$field[0]);
				$field[0] = ereg_replace("B\.",'',$field[0]);
				$field[0] = ereg_replace("C\.",'',$field[0]);
				$field[0] = ereg_replace("D\.",'',$field[0]);
				$values = split(';',$value);
				switch (strtoupper($field[0]))
				{
				case 'N':
					reset($this->names);
					$j=0;
					while(list($myname,$myval) = each($this->names) )
					{
						//$namel = 'per_' . $myname;
						$namel = $myname;
						$entry[$namel] = $values[$j];
						$j++;
					}
					break;
				case 'FN':
					$fn = split(" ", $values[0]);
					$entry['first_name'] = $fn[0];
					$entry['last_name'] = $fn[1];
					break;
				case 'TITLE':
					$entry['title'] = $values[0];
					break;
				case 'TZ':
					//$entry['tz'] = $values[0];
					break;
				case 'GEO':
					//$entry['geo'] = $values[0];
					break;
				case 'URL':
					//$entry['url'] = $values[0];
					$entry['comm_media']['website'] = $values[0];
					break;
				case 'NOTE':
					//$entry['note'] = ereg_replace('=0D=0A',"\n",$values[0]);
					$entry['notes']['type'] = 'vcard';
					$entry['notes']['note'] = ereg_replace('=0D=0A',"\n",$values[0]);
					break;
				case 'KEY':
					$entry['key'] = ereg_replace('=0D=0A',"\n",$values[0]);
					break;
				case 'LABEL':
					$entry['label'] = ereg_replace('=0D=0A',"\n",$values[0]);
					break;
				case 'BDAY': #1969-12-31
						     $tmp = split('-',$values[0]);
					if ($tmp[0])
					{
						$entry['birthday'] = $tmp[1] . '/' . $tmp[2] . '/' . $tmp[0];
					}
					break;
				case 'ORG':	// RB 2001/05/07 added for Lotus Organizer: ORG:Company;Department
					$entry['preferred_org'] = $values[0];
					$entry['department'] = $values[1];
					break;
				case 'ADR':
					switch ($field[1])
					{
					case 'INTL':
						$location_level = 'int';
						switch ($field[2])
						{
						case 'WORK':
							$location_type = 'work';
							break;
						case 'HOME':
							$location_type = 'home';
							break;
						default:
							break;
						}
						break;
					case 'DOM':
						$location_level = 'dom';
						switch ($field[2])
						{
						case 'WORK':
							$location_type = 'work';
							break;
						case 'HOME':
							$location_type = 'home';
							break;
						default:
							break;
						}
						break;
					case 'PARCEL':
						$location_level = 'parcel';
						switch ($field[2])
						{
						case 'WORK':
							$location_type = 'work';
							break;
						case 'HOME':
							$location_type = 'home';
							break;
						default:
							break;
						}
						break;
					case 'POSTAL':
						$location_level = 'postal';
						switch ($field[2])
						{
						case 'WORK':
							$location_type = 'work';
							break;
						case 'HOME':
							$location_type = 'home';
							break;
						default:
							break;
						}
						break;
					case 'WORK':
						$location_type = 'work';
						break;
					case 'HOME':
						$location_type = 'home';
						break;
					default:
						$location_type = 'work';
						break;
					}
					$loc = $location_level?$location_level.'_'.$location_type:$location_type;
					$entry['locations'][$loc]['type'] = $location_type;
					$entry['locations'][$loc]['add1'] = $values[1];
					$entry['locations'][$loc]['add2'] = $values[2];
					$entry['locations'][$loc]['city'] = $values[3];
					$entry['locations'][$loc]['state'] = $values[4];
					$entry['locations'][$loc]['postal_code'] = $values[5];
					$entry['locations'][$loc]['country'] = $values[6];
					break;
				case 'TEL': 	
					// RB 2001/05/07 added for Lotus Organizer ueses TEL;{WORK|HOME};{VOICE|FAX}[;PREF]
					if ($field[2] == 'FAX' && ($field[1] == 'WORK' || $field[1] == 'HOME'))
					{
						/* TODO This is PHP4 only */
						array_shift($field);	// --> ignore the WORK or HOME if FAX follows, HOME;FAX and HOME;TEL are maped later
					}
					switch ($field[1])
					{
					case 'WORK':	
						// RB don't overwrite TEL;WORK;VOICE (main nr.) with TEL;WORK, TEL;WORK --> tel_isdn
						//$entry[$buffer['tel_work'] ? 'tel_isdn' : 'tel_work'] = $values[0];
						$entry['comm_media']['work phone'] = $values[0];
						break;
					case 'HOME':	
						// RB don't overwrite TEL;HOME;VOICE (main nr.) with TEL;HOME, TEL;HOME --> ophone
						//$entry[$buffer['tel_home'] ? 'ophone' : 'tel_home' ] = $values[0];
						$entry['comm_media']['home phone'] = $values[0];
						break;
					case 'VOICE':
						$entry['comm_media']['voice phone'] = $values[0];
						break;
					case 'FAX':
						switch ($fields[2])
						{
						case 'WORK':
							if(!$entry['comm_media']['work fax'])
							{
								$entry['comm_media']['work fax'] = $values[0];
							}
							else
							{
								$entry['comm_media']['other work fax'] = $values[0];
							}
							break;
						case 'HOME':
							$entry['comm_media']['home fax'] = $values[0];
							break;
						default:
							$entry['comm_media']['work fax'] = $values[0];
						}
						break;
					case 'MSG':
						$entry['comm_media']['msg phone'] = $values[0];
						break;
					case 'CELL':
						$entry['comm_media']['mobile (cell) phone'] = $values[0];
						break;
					case 'PAGER':
						$entry['comm_media']['pager'] = $values[0];
						break;
					case 'BBS':
						$entry['comm_media']['bbs'] = $values[0];
						break;
					case 'MODEM':
						$entry['comm_media']['modem'] = $values[0];
						break;
					case 'CAR':
						$entry['comm_media']['car phone'] = $values[0];
						break;
					case 'ISDN':
						$entry['comm_media']['isdn'] = $values[0];
						break;
					case 'VIDEO':
						$entry['comm_media']['video'] = $values[0];
						break;
					case 'PREF':
						//echo $field[2].' is preferred';
						if ($field[2])
						{
							//$buffer['tel_prefer'] .= strtolower($field[2]) . ';';
							$entry['comm_media_preferred'] .= strtolower($field[2]) . ';';
						}
						break;
					default:
						break;
					}
					if ($field[2] == 'PREF')
					{
						$entry['comm_media_preferred'] .= strtolower($field[1]) . ';';
					}
					break;
				case 'EMAIL':
					switch ($field[1])
					{
					case 'WORK':
						$entry['comm_media']['work email'] = $values[0];
						break;
					case 'HOME':
						$entry['comm_media']['home email'] = $values[0];
						break;
					default:
						if($buffer['email'])
						{
							$entry['comm_media']['work email'] = $values[2];
							//$entry['email_type'] = $field[2];
						}
						elseif (!$buffer['email'])
						{
							$entry['comm_media']['work email'] = $values[0];
						}
						break;
					}
					break;
				default:
					break;
				}
			}
			
			if (count($street = split("\r*\n",$buffer['adr_one_street'],3)) > 1)
			{
				$entry['adr_one_street'] = $street[0];			// RB 2001/05/08 added for Lotus Organizer to split multiline adresses
				$entry['address2']		  = $street[1];
				$entry['address3']		  = $street[2];
			}
			return $entry;
		}

		// Takes an array of contacts class fields/values, turns it into a vcard string:
		//
		// for ($i=0;$i<count($buffer);$i++) {
		//     $vcards .= $this->vcard->out($buffer[$i]);
		// }
		//
		function out($buffer)
		{
			$entry   = '';
			$header  = 'BEGIN:VCARD' . "\n";
			$header .= 'VERSION:2.1' . "\n";
			$header .= 'X-PHPGROUPWARE-FILE-AS:phpGroupWare.org' . "\n";

			reset($this->export);
			while ( list($name,$value) = each($this->export) )
			{
				if (!empty($buffer[$value]))
				{
					$mult = explode(';',$value);
					if (!$mult[1])
					{ // Normal
						if (strstr($buffer[$value],"\r\n") || strstr($buffer[$value],"\n"))
						{
							$buffer[$value] = ereg_replace("\r\n","=0D=0A",$buffer[$value]);
							$buffer[$value] = ereg_replace("\n","=0D=0A",$buffer[$value]);
							$entry .= $value . ';QUOTED-PRINTABLE:' . $buffer[$value]."\n";
						}
						elseif ($value == 'BDAY')
						{
							$tmp = split('/',$buffer[$value]); # 12/31/1969 -> 1969-12-31
							if ($tmp[0])
							{
								if (strlen($tmp[0]) == 1) { $tmp[0] = '0'.$tmp[0]; }
								if (strlen($tmp[0]) == 1) { $tmp[1] = '0'.$tmp[1]; }
								$entry .= 'BDAY:' . $tmp[2] . '-' . $tmp[0] . '-' . $tmp[1] . "\n";
							}
						}
						else
						{
							$entry .= $value . ':' . $buffer[$value] . "\n";
						}
					}
					else
					{
						switch ($mult[0])
						{
							case 'N':
								switch ($mult[1])
								{
									case 'PREFIX':
										$prefix    = ';' . $buffer[$value];
										break;
									case 'GIVEN':
										$firstname = ';' . $buffer[$value];
										break;
									case 'MIDDLE':
										$middle    = ';' . $buffer[$value];
										break;
									case 'FAMILY':
										$lastname  =       $buffer[$value];
										break;
									case 'SUFFIX':
										$suffix    = ';' . $buffer[$value];
										break;
								}
								break;
							case 'ORG':
								switch ($mult[1])
								{
									case 'NAME':
										$org_name = $buffer[$value];
										break;
									case 'UNIT':
										$org_unit = ';' . $buffer[$value];
										break;
								}
								break;
							case 'ADR':
								switch ($mult[1])
								{
									case 'TYPE':
										$types = explode(';',$buffer[$value]);
										if ($types[1])
										{
											while ( $type = each($types) )
											{
												$typei[$i][$mult[2]] .= ';' . strtoupper($type[1]);
											}
										}
										elseif ($types[0])
										{
											$typei[$i][$mult[2]] .= ';' . strtoupper($types[0]);
										}
										else
										{
											$typei[$i][$mult[2]] .= ';' . strtoupper($buffer[$value]);
										}
										//echo "TYPE=".$typei[$i][$mult[2]];
										break;
									case 'WORK':
										$workaddr .= $buffer[$value] . ';';
										$workattr = $mult[0] . ';' . $mult[1] . $typei[$i][$mult[1]];
										break;
									case 'HOME':
										$homeaddr .= $buffer[$value] . ';';
										$homeattr = $mult[0] . ';' . $mult[1] . $typei[$i][$mult[1]];
										break;
									default:
										break;
								}
								break;
							case 'TEL':
								switch($mult[1])
								{
									case 'PREFER':
										$prefer = explode(';',$buffer[$value]);
										if ($prefer[1])
										{
											while ($pref = strtoupper(each($prefer)))
											{
												$prefi[$i][$pref] = ';PREF';
											}
											//echo 'PREF1';
										}
										elseif ($prefer[0])
										{
											$prefi[$i][strtoupper($prefer[0])] = ';PREF';
											//echo 'PREF='.strtoupper($prefer[0]);
										}
										elseif ($buffer[$value])
										{
											$prefi[$i][$buffer[$value]] = ';PREF';
											//echo 'PREF3';
										}
										break;
									case 'WORK':
										// Wow, this is fun!
										$entry .= 'A.' . $mult[0] . ';' . $mult[1] . $prefi[$i][$mult[1]] . ':' . $buffer[$value] . "\n";
										break;
									case 'HOME':
										$entry .= 'B.' . $mult[0] . ';' . $mult[1] . $prefi[$i][$mult[1]] . ':' . $buffer[$value] . "\n";
										break;
									default:
										$entry .= $mult[0] . ';' . $mult[1] . $prefi[$i][$mult[1]] . ':' . $buffer[$value] . "\n";
										break;
								}
								break;
							case 'EMAIL':
								switch($mult[1])
								{
									case 'TYPE':
										if ($mult[2] == 'WORK') { $emailtype  = ';' . $buffer[$value]; }
										if ($mult[2] == 'HOME') { $hemailtype = ';' . $buffer[$value]; }
										break;
									case 'WORK':
										$newval = 'A.'.$value;
										$entry .= $newval . $emailtype . ':' . $buffer[$value] . "\n";
										break;
									case 'HOME':
										$newval = 'B.' . $value;
										$entry .= $newval . $hemailtype . ':' . $buffer[$value] . "\n";
										break;
									default:
										break;
								}
								break;
							default:
								break;
						} //end switch ($mult[0])
					} //end else
				} //end if (!empty)
			} //end while

			if ($lastname && $firstname)
			{
				$entries .= $header;
				$entries .= 'N:' . $lastname . $firstname . $middle . $prefix . $suffix . "\n";
				$entries .= $entry;

				if (!$buffer['FN'])
				{
					if ($lastname || $firstname )
					{
						$entries .= 'FN:' . substr($firstname,1) . ' ' . $lastname . "\n";
					}
				}
				if ($org_name || $org_unit)
				{
					$entries .= 'ORG:' . $org_name . $org_unit . "\n";
				}

				$workattr = ereg_replace('ADR;','',$workattr);
				$homeattr = ereg_replace('ADR;','',$homeattr);
				if (!$buffer['EXT']) { $buffer['EXT'] = ';'; }
				if ($workaddr)
				{
					$work = 'A.ADR;' . $workattr . ':;' . $buffer['EXT'] . substr($workaddr,0,-1) . "\n";
					if (!$buffer['LABEL'])
					{
						$wlabel = substr($workaddr,0,-1);
						$wlabel = ereg_replace(';','=0D=0A',$wlabel);
						//$wlabel = ereg_replace('(',',',$wlabel);
						//$wlabel = ereg_replace(')',',',$wlabel);
						$wlabel = 'LABEL;WORK;QUOTED-PRINTABLE:' . $wlabel . "\n";
					}
				}
				if ($homeaddr)
				{
					$home = 'B.ADR;'.$homeattr.':;;'.substr($homeaddr,0,-1)."\n";
					$hlabel = substr($homeaddr,0,-1);
					$hlabel = ereg_replace(';','=0D=0A',$hlabel);
					//$hlabel = ereg_replace('(',',',$hlabel);
					//$hlabel = ereg_replace(')',',',$hlabel);
					$hlabel = 'LABEL;HOME;QUOTED-PRINTABLE:' . $hlabel . "\n";
				}
				$entries = ereg_replace('PUBKEY','KEY',$entries);
				$entries .= $work . $home . $wlabel . $hlabel . 'END:VCARD' . "\n";
				$entries .= "\n";

				$buffer = $entries;
				return $buffer;
			}
			else
			{
				return;
			}
		} //end function
	} //end class
?>
