<?php
	 /**************************************************************************\
	 * phpGroupWare API - Logging helper functions                              *
	 * This file written by Doug Dicks <doug@revelanttech.com>                  *
	 * This is just an alternative API to the logging methods in                *
	 * class.errorlog2.inc.php.  They allow you to call the logging methods     *
	 * without having to always use a  $GLOBALS['phpgw']->log-> with all of     *
	 * your logging statements.  For example, instead of                        *
	 * $GLOBALS['phpgw']->log->debug(); you can use log_debug();                *
	 *                                                                          *
	 * The goal is to make it easier to add logging to your code.               *
	 * -------------------------------------------------------------------------*
	 * This library is part of the phpGroupWare API                             *
	 * http://www.phpgroupware.org/api                                          * 
	 * ------------------------------------------------------------------------ *
	 * This library is free software; you can redistribute it and/or modify it  *
	 * under the terms of the GNU General Public License as published by        *
	 * the Free Software Foundation; either version 2.1 of the License,         *
	 * or any later version.                                                    *
	 * This library is distributed in the hope that it will be useful, but      *
	 * WITHOUT ANY WARRANTY; without even the implied warranty of               *
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
	 * See the GNU General Public License for more details.                     *
	 * You should have received a copy of the GNU General Public License        *
	 * along with this library; if not, write to the Free Software Foundation,  *
	 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA            *
	 \**************************************************************************/

	/* $Id: log_functions.inc.php,v 1.1.2.1 2003/10/20 13:04:34 cw Exp $ */

	/*!
	 @function log_debug
	 @abstract Log a message at DEBUG level
	 @author doug
	 @discussion Log a message at DEBUG level
	 @syntax 
	 @example 
		function somefunc()
		{
			/// some code...

			log_debug(array('text' => 'Did something, now X = %1',
						    'p1' => $x,
						 	'file' => __FILE__,
						 	'line' => __LINE__));

			// some more code...
			// This syntax also works:
			log_debug('Did something else, now X = %1', $x);
			// or even if there are no parameters:
			log_debug('Did something else');
		}
	*/
	function log_debug()
	{
		if ( isset($GLOBALS['phpgw']->log) )
		{
			$arg_array = func_get_args();
			$GLOBALS['phpgw']->log->log_if_level('D', $GLOBALS['phpgw']->log->make_parms($arg_array));
		}
	}
	
	function log_info()
	{
		if ( isset($GLOBALS['phpgw']->log) )
		{
			$arg_array = func_get_args();
			$GLOBALS['phpgw']->log->log_if_level('I', $GLOBALS['phpgw']->log->make_parms($arg_array));
		}
	}

	function log_warn()
	{
		if ( isset($GLOBALS['phpgw']->log) )
		{
		$arg_array = func_get_args();
		$GLOBALS['phpgw']->log->log_if_level('W', $GLOBALS['phpgw']->log->make_parms($arg_array));
		}
	}

	function log_error()
	{
		if ( isset($GLOBALS['phpgw']->log) )
		{
			$arg_array = func_get_args();
			$GLOBALS['phpgw']->log->log_if_level('E', $GLOBALS['phpgw']->log->make_parms($arg_array));
		}
	}

	function log_fatal()
	{
		if ( isset($GLOBALS['phpgw']->log) )
		{
			$arg_array = func_get_args();
			$GLOBALS['phpgw']->log->log_if_level('F', $GLOBALS['phpgw']->log->make_parms($arg_array));
		}
	}
	
	
	/* For backward compatibility with some of the existing debugging statements */
	function print_debug($message,$var = 'messageonly',$part = 'app', $level = 3)
	{
		if ( $var == 'messageonly' )
		{
			log_debug($message);
		}
		else
		{
			log_debug($message, $var);
		}
	}
	
?>
